#! python

# The script queries BDII for gsiftp service endpoint. Background of this command:
#
# https://savannah.cern.ch/bugs/?65817
#
# It is a temporary workaround: glite-sd-query cannot return gsiftp endpoints.

import ldap
import os
import re

def getServiceInfo(entry):

    endpoint_attr = 'GlueSEAccessProtocolEndpoint'

    if endpoint_attr in entry:
        endpoint = entry[endpoint_attr][0]
    else:
       return []

    version_attr = 'GlueSEAccessProtocolVersion'

    if version_attr in entry:
        version = entry[version_attr][0]
    else:
        return []

    type_attr = 'GlueSEAccessProtocolType'

    if type_attr in entry:
        type = entry[type_attr][0]
    else:
        return []

    return [endpoint, version, type]


def getVOInfo(entry):

    vo_attr = 'GlueSAAccessControlBaseRule'    
    volist = []

    if vo_attr in entry:  
        volist_tmp = entry[vo_attr]

        for i in volist_tmp:
            vo = i.replace("vo:", "")
            vo = vo.replace("VO:", "")
            volist.append(vo)

    return volist


def getSiteInfo(entry):

    site_attr = 'GlueForeignKey'
    site = ""

    if site_attr in entry:  
        site = entry[site_attr][0].split("=")[1]

    return site


parameter_bdiiHost = os.environ['LCG_GFAL_INFOSYS']
parameter_base_dn = 'o=grid'

ldap_connaction = ldap.initialize("ldap://%s" % parameter_bdiiHost)
ldap_connaction.simple_bind_s()

filter = "GlueSEImplementationName=classicSE"
res = ldap_connaction.search_s(parameter_base_dn, ldap.SCOPE_SUBTREE, filter)
services = {}

for dn,entry in res:
    id = entry["GlueSEUniqueID"][0]
    chunkKey = "GlueSEUniqueID=%s" % id
    filter = "GlueChunkKey=%s" % chunkKey
    res_se = ldap_connaction.search_s(parameter_base_dn, ldap.SCOPE_SUBTREE, filter)
    voList = []
    serviceInfo = []
    siteInfo = getSiteInfo(entry)

    for dn_se,entry_se in res_se:
        if re.search("GlueSALocalID", dn_se):
            voList = getVOInfo(entry_se)
        elif re.search("GlueSEAccessProtocolLocalID" ,dn_se): 
            serviceInfo = getServiceInfo(entry_se)

    if len(voList) and len(siteInfo) and len(serviceInfo) == 3:
        se = {'endpoint' : serviceInfo[0], 'site' : siteInfo, 'version' : serviceInfo[1], 'type': serviceInfo[2], 'volist' : voList}
        services[id] = se

for id, entry in services.items():
    print '<service name="%s">' % entry['endpoint']
    print '    <parameters>'
    print '        <endpoint>%s</endpoint>' % entry['endpoint']
    print '        <type>%s</type>' % entry['type']
    print '        <site>%s</site>' % entry['site']
    print '        <version>%s</version>' % entry['version']
    print '        <volist>'

    for vo in entry['volist']:
        print '            <vo>%s</vo>' % vo
           
    print '        </volist>'
    print '    </parameters>'
    print '</service>'

